@@ -1,27 +1,28 @@
-import sys
+from email.parser import FeedParser
 import os
-import shutil
-import re
-import zipfile
+import imp
 import pkg_resources
+import re
+import sys
+import shutil
 import tempfile
-import urlparse
-import urllib2
-import urllib
-import ConfigParser
-from distutils.sysconfig import get_python_version
-from email.FeedParser import FeedParser
+import zipfile
+
 from pip.locations import bin_py, running_under_virtualenv
-from pip.exceptions import InstallationError, UninstallationError
+from pip.exceptions import (InstallationError, UninstallationError,
+                            BestVersionAlreadyInstalled,
+                            DistributionNotFound)
 from pip.vcs import vcs
 from pip.log import logger
 from pip.util import display_path, rmtree
-from pip.util import ask, backup_dir
-from pip.util import is_installable_dir, is_local, dist_is_local
-from pip.util import renames, normalize_path, egg_link_path
+from pip.util import ask, ask_path_exists, backup_dir
+from pip.util import is_installable_dir, is_local, dist_is_local, dist_in_usersite
+from pip.util import renames, normalize_path, egg_link_path, dist_in_site_packages
 from pip.util import make_path_relative
-from pip import call_subprocess
-from pip.backwardcompat import any, copytree
+from pip.util import call_subprocess
+from pip.backwardcompat import (urlparse, urllib, uses_pycache,
+                                ConfigParser, string_types, HTTPError,
+                                get_python_version, b)
 from pip.index import Link
 from pip.locations import build_prefix
 from pip.download import (get_file_content, is_url, url_to_path,
@@ -36,14 +37,17 @@ PIP_DELETE_MARKER_FILENAME = 'pip-delete-this-directory.txt'
 class InstallRequirement(object):
 
     def __init__(self, req, comes_from, source_dir=None, editable=False,
-                 url=None, update=True):
-        if isinstance(req, basestring):
+                 url=None, as_egg=False, update=True):
+        self.extras = ()
+        if isinstance(req, string_types):
             req = pkg_resources.Requirement.parse(req)
+            self.extras = req.extras
         self.req = req
         self.comes_from = comes_from
         self.source_dir = source_dir
         self.editable = editable
         self.url = url
+        self.as_egg = as_egg
         self._egg_info_path = None
         # This holds the pkg_resources.Distribution object if this requirement
         # is already available:
@@ -59,15 +63,22 @@ class InstallRequirement(object):
         self.install_succeeded = None
         # UninstallPathSet of uninstalled distribution (for possible rollback)
         self.uninstalled = None
+        self.use_user_site = False
 
     @classmethod
     def from_editable(cls, editable_req, comes_from=None, default_vcs=None):
-        name, url = parse_editable(editable_req, default_vcs)
+        name, url, extras_override = parse_editable(editable_req, default_vcs)
         if url.startswith('file:'):
             source_dir = url_to_path(url)
         else:
             source_dir = None
-        return cls(name, comes_from, source_dir=source_dir, editable=True, url=url)
+
+        res = cls(name, comes_from, source_dir=source_dir, editable=True, url=url)
+
+        if extras_override is not None:
+            res.extras = extras_override
+
+        return res
 
     @classmethod
     def from_line(cls, name, comes_from=None):
@@ -76,28 +87,34 @@ class InstallRequirement(object):
         """
         url = None
         name = name.strip()
-        req = name
+        req = None
         path = os.path.normpath(os.path.abspath(name))
+        link = None
 
         if is_url(name):
-            url = name
-            ## FIXME: I think getting the requirement here is a bad idea:
-            #req = get_requirement_from_url(url)
-            req = None
+            link = Link(name)
         elif os.path.isdir(path) and (os.path.sep in name or name.startswith('.')):
             if not is_installable_dir(path):
-                raise InstallationError("Directory %r is not installable. File 'setup.py' not found."
-                                        % name)
-            url = path_to_url(name)
-            #req = get_requirement_from_url(url)
-            req = None
+                raise InstallationError("Directory %r is not installable. File 'setup.py' not found.", name)
+            link = Link(path_to_url(name))
         elif is_archive_file(path):
             if not os.path.isfile(path):
-                logger.warn('Requirement %r looks like a filename, but the file does not exist'
-                            % name)
-            url = path_to_url(name)
-            #req = get_requirement_from_url(url)
-            req = None
+                logger.warn('Requirement %r looks like a filename, but the file does not exist', name)
+            link = Link(path_to_url(name))
+
+        # If the line has an egg= definition, but isn't editable, pull the requirement out.
+        # Otherwise, assume the name is the req for the non URL/path/archive case.
+        if link and req is None:
+            url = link.url_without_fragment
+            req = link.egg_fragment
+
+            # Handle relative file URLs
+            if link.scheme == 'file' and re.search(r'\.\./', url):
+                url = path_to_url(os.path.normpath(os.path.abspath(link.path)))
+
+        else:
+            req = name
+
         return cls(req, comes_from, url=url)
 
     def __str__(self):
@@ -110,7 +127,7 @@ class InstallRequirement(object):
         if self.satisfied_by is not None:
             s += ' in %s' % display_path(self.satisfied_by.location)
         if self.comes_from:
-            if isinstance(self.comes_from, basestring):
+            if isinstance(self.comes_from, string_types):
                 comes_from = self.comes_from
             else:
                 comes_from = self.comes_from.from_path()
@@ -123,7 +140,7 @@ class InstallRequirement(object):
             return None
         s = str(self.req)
         if self.comes_from:
-            if isinstance(self.comes_from, basestring):
+            if isinstance(self.comes_from, string_types):
                 comes_from = self.comes_from
             else:
                 comes_from = self.comes_from.from_path()
@@ -229,17 +246,19 @@ class InstallRequirement(object):
     _run_setup_py = """
 __file__ = __SETUP_PY__
 from setuptools.command import egg_info
+import pkg_resources
+import os
 def replacement_run(self):
     self.mkpath(self.egg_info)
     installer = self.distribution.fetch_build_egg
-    for ep in egg_info.iter_entry_points('egg_info.writers'):
+    for ep in pkg_resources.iter_entry_points('egg_info.writers'):
         # require=False is the change we're making:
         writer = ep.load(require=False)
         if writer:
-            writer(self, ep.name, egg_info.os.path.join(self.egg_info,ep.name))
+            writer(self, ep.name, os.path.join(self.egg_info,ep.name))
     self.find_sources()
 egg_info.egg_info.run = replacement_run
-execfile(__file__)
+exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))
 """
 
     def egg_info_data(self, filename):
@@ -269,7 +288,10 @@ execfile(__file__)
                     for dir in vcs.dirnames:
                         if dir in dirs:
                             dirs.remove(dir)
-                    for dir in dirs:
+                    # Iterate over a copy of ``dirs``, since mutating
+                    # a list while iterating over it can cause trouble.
+                    # (See https://github.com/pypa/pip/pull/462.)
+                    for dir in list(dirs):
                         # Don't search in anything that looks like a virtualenv environment
                         if (os.path.exists(os.path.join(root, dir, 'bin', 'python'))
                             or os.path.exists(os.path.join(root, dir, 'Scripts', 'Python.exe'))):
@@ -324,11 +346,12 @@ execfile(__file__)
     def requirements(self, extras=()):
         in_extra = None
         for line in self.egg_info_lines('requires.txt'):
-            match = self._requirements_section_re.match(line)
+            match = self._requirements_section_re.match(line.lower())
             if match:
                 in_extra = match.group(1)
                 continue
             if in_extra and in_extra not in extras:
+                logger.debug('skipping extra %s' % in_extra)
                 # Skip requirement for an extra we aren't requiring
                 continue
             yield line
@@ -345,18 +368,9 @@ execfile(__file__)
 
     def assert_source_matches_version(self):
         assert self.source_dir
-        if self.comes_from is None:
-            # We don't check the versions of things explicitly installed.
-            # This makes, e.g., "pip Package==dev" possible
-            return
         version = self.installed_version
         if version not in self.req:
-            logger.fatal(
-                'Source in %s has the version %s, which does not match the requirement %s'
-                % (display_path(self.source_dir), version, self))
-            raise InstallationError(
-                'Source in %s has version %s that conflicts with %s'
-                % (display_path(self.source_dir), version, self))
+            logger.warn('Requested %s, but installing version %s' % (self, self.installed_version))
         else:
             logger.debug('Source in %s has version %s, which satisfies requirement %s'
                          % (display_path(self.source_dir), version, self))
@@ -407,16 +421,28 @@ execfile(__file__)
 
         pip_egg_info_path = os.path.join(dist.location,
                                          dist.egg_name()) + '.egg-info'
+        # workaround for http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=618367
+        debian_egg_info_path = pip_egg_info_path.replace(
+            '-py%s' % pkg_resources.PY_MAJOR, '')
         easy_install_egg = dist.egg_name() + '.egg'
         develop_egg_link = egg_link_path(dist)
-        if os.path.exists(pip_egg_info_path):
+
+        pip_egg_info_exists = os.path.exists(pip_egg_info_path)
+        debian_egg_info_exists = os.path.exists(debian_egg_info_path)
+        if pip_egg_info_exists or debian_egg_info_exists:
             # package installed by pip
-            paths_to_remove.add(pip_egg_info_path)
+            if pip_egg_info_exists:
+                egg_info_path = pip_egg_info_path
+            else:
+                egg_info_path = debian_egg_info_path
+            paths_to_remove.add(egg_info_path)
             if dist.has_metadata('installed-files.txt'):
                 for installed_file in dist.get_metadata('installed-files.txt').splitlines():
-                    path = os.path.normpath(os.path.join(pip_egg_info_path, installed_file))
+                    path = os.path.normpath(os.path.join(egg_info_path, installed_file))
                     paths_to_remove.add(path)
-            if dist.has_metadata('top_level.txt'):
+            #FIXME: need a test for this elif block
+            #occurs with --single-version-externally-managed/--record outside of pip
+            elif dist.has_metadata('top_level.txt'):
                 if dist.has_metadata('namespace_packages.txt'):
                     namespaces = dist.get_metadata('namespace_packages.txt')
                 else:
@@ -436,7 +462,7 @@ execfile(__file__)
                                             'easy-install.pth')
             paths_to_remove.add_pth(easy_install_pth, './' + easy_install_egg)
 
-        elif os.path.isfile(develop_egg_link):
+        elif develop_egg_link:
             # develop egg
             fh = open(develop_egg_link, 'r')
             link_pointer = os.path.normcase(fh.readline().strip())
@@ -489,8 +515,9 @@ execfile(__file__)
         archive_name = '%s-%s.zip' % (self.name, self.installed_version)
         archive_path = os.path.join(build_dir, archive_name)
         if os.path.exists(archive_path):
-            response = ask('The file %s exists. (i)gnore, (w)ipe, (b)ackup '
-                           % display_path(archive_path), ('i', 'w', 'b'))
+            response = ask_path_exists(
+                'The file %s exists. (i)gnore, (w)ipe, (b)ackup ' %
+                display_path(archive_path), ('i', 'w', 'b'))
             if response == 'i':
                 create_archive = False
             elif response == 'w':
@@ -511,7 +538,7 @@ execfile(__file__)
                     dirname = os.path.join(dirpath, dirname)
                     name = self._clean_zip_name(dirname, dir)
                     zipdir = zipfile.ZipInfo(self.name + '/' + name + '/')
-                    zipdir.external_attr = 0755 << 16L
+                    zipdir.external_attr = 0x1ED << 16 # 0o755
                     zip.writestr(zipdir, '')
                 for filename in filenames:
                     if filename == PIP_DELETE_MARKER_FILENAME:
@@ -534,19 +561,21 @@ execfile(__file__)
         if self.editable:
             self.install_editable(install_options, global_options)
             return
+
         temp_location = tempfile.mkdtemp('-record', 'pip-')
         record_filename = os.path.join(temp_location, 'install-record.txt')
         try:
-
             install_args = [
                 sys.executable, '-c',
                 "import setuptools;__file__=%r;"\
-                "execfile(__file__)" % self.setup_py] +\
+                "exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))" % self.setup_py] +\
                 list(global_options) + [
                 'install',
-                '--single-version-externally-managed',
                 '--record', record_filename]
 
+            if not self.as_egg:
+                install_args += ['--single-version-externally-managed']
+
             if running_under_virtualenv():
                 ## FIXME: I'm not sure if this is a reasonable location; probably not
                 ## but we can't put it in the default location, as that is a virtualenv symlink that isn't writable
@@ -564,6 +593,11 @@ execfile(__file__)
                 logger.notify('Record file %s not found' % record_filename)
                 return
             self.install_succeeded = True
+            if self.as_egg:
+                # there's no --always-unzip option we can pass to install command
+                # so we unable to save the installed-files.txt
+                return
+
             f = open(record_filename)
             for line in f:
                 line = line.strip()
@@ -611,7 +645,7 @@ execfile(__file__)
             ## FIXME: should we do --install-headers here too?
             call_subprocess(
                 [sys.executable, '-c',
-                 "import setuptools; __file__=%r; execfile(%r)" % (self.setup_py, self.setup_py)]
+                 "import setuptools; __file__=%r; exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))" % self.setup_py]
                 + list(global_options) + ['develop', '--no-deps'] + list(install_options),
 
                 cwd=self.source_dir, filter_stdout=self._filter_install,
@@ -643,7 +677,15 @@ execfile(__file__)
         except pkg_resources.DistributionNotFound:
             return False
         except pkg_resources.VersionConflict:
-            self.conflicts_with = pkg_resources.get_distribution(self.req.project_name)
+            existing_dist = pkg_resources.get_distribution(self.req.project_name)
+            if self.use_user_site:
+                if dist_in_usersite(existing_dist):
+                    self.conflicts_with = existing_dist
+                elif running_under_virtualenv() and dist_in_site_packages(existing_dist):
+                    raise InstallationError("Will not install to the user site because it will lack sys.path precedence to %s in %s"
+                                            %(existing_dist.project_name, existing_dist.location))
+            else:
+                self.conflicts_with = existing_dist
         return True
 
     @property
@@ -739,38 +781,36 @@ class Requirements(object):
         return self._keys
 
     def values(self):
-        values_list = []
-        for key in self._keys:
-            values_list.append(self._dict[key])
-        return values_list
+        return [self._dict[key] for key in self._keys]
 
     def __contains__(self, item):
         return item in self._keys
-    
+
     def __setitem__(self, key, value):
         if key not in self._keys:
             self._keys.append(key)
         self._dict[key] = value
-    
+
     def __getitem__(self, key):
         return self._dict[key]
 
     def __repr__(self):
-        values = [ '%s: %s' % (repr(k), repr(self[k])) for k in self.keys() ]
+        values = ['%s: %s' % (repr(k), repr(self[k])) for k in self.keys()]
         return 'Requirements({%s})' % ', '.join(values)
 
 
 class RequirementSet(object):
 
     def __init__(self, build_dir, src_dir, download_dir, download_cache=None,
-                 upgrade=False, ignore_installed=False,
-                 ignore_dependencies=False):
+                 upgrade=False, ignore_installed=False, as_egg=False,
+                 ignore_dependencies=False, force_reinstall=False, use_user_site=False):
         self.build_dir = build_dir
         self.src_dir = src_dir
         self.download_dir = download_dir
         self.download_cache = download_cache
         self.upgrade = upgrade
         self.ignore_installed = ignore_installed
+        self.force_reinstall = force_reinstall
         self.requirements = Requirements()
         # Mapping of alias: real_name
         self.requirement_aliases = {}
@@ -779,6 +819,8 @@ class RequirementSet(object):
         self.successfully_downloaded = []
         self.successfully_installed = []
         self.reqs_to_cleanup = []
+        self.as_egg = as_egg
+        self.use_user_site = use_user_site
 
     def __str__(self):
         reqs = [req for req in self.requirements.values()
@@ -788,6 +830,8 @@ class RequirementSet(object):
 
     def add_requirement(self, install_req):
         name = install_req.name
+        install_req.as_egg = self.as_egg
+        install_req.use_user_site = self.use_user_site
         if not name:
             self.unnamed_requirements.append(install_req)
         else:
@@ -808,7 +852,7 @@ class RequirementSet(object):
 
     @property
     def has_requirements(self):
-        return self.requirements.values() or self.unnamed_requirements
+        return list(self.requirements.values()) or self.unnamed_requirements
 
     @property
     def has_editables(self):
@@ -845,10 +889,10 @@ class RequirementSet(object):
             req.commit_uninstall()
 
     def locate_files(self):
-        ## FIXME: duplicates code from install_files; relevant code should
+        ## FIXME: duplicates code from prepare_files; relevant code should
         ##        probably be factored out into a separate method
         unnamed = list(self.unnamed_requirements)
-        reqs = self.requirements.values()
+        reqs = list(self.requirements.values())
         while reqs or unnamed:
             if unnamed:
                 req_to_install = unnamed.pop(0)
@@ -884,25 +928,45 @@ class RequirementSet(object):
     def prepare_files(self, finder, force_root_egg_info=False, bundle=False):
         """Prepare process. Create temp directories, download and/or unpack files."""
         unnamed = list(self.unnamed_requirements)
-        reqs = self.requirements.values()
+        reqs = list(self.requirements.values())
         while reqs or unnamed:
             if unnamed:
                 req_to_install = unnamed.pop(0)
             else:
                 req_to_install = reqs.pop(0)
             install = True
+            best_installed = False
+            not_found = None
             if not self.ignore_installed and not req_to_install.editable:
                 req_to_install.check_if_exists()
                 if req_to_install.satisfied_by:
                     if self.upgrade:
-                        req_to_install.conflicts_with = req_to_install.satisfied_by
-                        req_to_install.satisfied_by = None
+                        if not self.force_reinstall and not req_to_install.url:
+                            try:
+                                url = finder.find_requirement(
+                                    req_to_install, self.upgrade)
+                            except BestVersionAlreadyInstalled:
+                                best_installed = True
+                                install = False
+                            except DistributionNotFound:
+                                not_found = sys.exc_info()[1]
+                            else:
+                                # Avoid the need to call find_requirement again
+                                req_to_install.url = url.url
+
+                        if not best_installed:
+                            req_to_install.conflicts_with = req_to_install.satisfied_by
+                            req_to_install.satisfied_by = None
                     else:
                         install = False
                 if req_to_install.satisfied_by:
-                    logger.notify('Requirement already satisfied '
-                                  '(use --upgrade to upgrade): %s'
-                                  % req_to_install)
+                    if best_installed:
+                        logger.notify('Requirement already up-to-date: %s'
+                                      % req_to_install)
+                    else:
+                        logger.notify('Requirement already satisfied '
+                                      '(use --upgrade to upgrade): %s'
+                                      % req_to_install)
             if req_to_install.editable:
                 logger.notify('Obtaining %s' % req_to_install)
             elif install:
@@ -933,12 +997,17 @@ class RequirementSet(object):
                     ##occurs when the script attempts to unpack the
                     ##build directory
 
+                    # NB: This call can result in the creation of a temporary build directory
                     location = req_to_install.build_location(self.build_dir, not self.is_download)
+
                     ## FIXME: is the existance of the checkout good enough to use it?  I don't think so.
                     unpack = True
+                    url = None
                     if not os.path.exists(os.path.join(location, 'setup.py')):
                         ## FIXME: this won't upgrade when there's an existing package unpacked in `location`
                         if req_to_install.url is None:
+                            if not_found:
+                                raise not_found
                             url = finder.find_requirement(req_to_install, upgrade=self.upgrade)
                         else:
                             ## FIXME: should req_to_install.url already be a link?
@@ -947,7 +1016,8 @@ class RequirementSet(object):
                         if url:
                             try:
                                 self.unpack_url(url, location, self.is_download)
-                            except urllib2.HTTPError, e:
+                            except HTTPError:
+                                e = sys.exc_info()[1]
                                 logger.fatal('Could not install requirement %s because of error %s'
                                              % (req_to_install, e))
                                 raise InstallationError(
@@ -957,7 +1027,6 @@ class RequirementSet(object):
                             unpack = False
                     if unpack:
                         is_bundle = req_to_install.is_bundle
-                        url = None
                         if is_bundle:
                             req_to_install.move_bundle_files(self.build_dir, self.src_dir)
                             for subreq in req_to_install.bundle_requirements():
@@ -965,8 +1034,8 @@ class RequirementSet(object):
                                 self.add_requirement(subreq)
                         elif self.is_download:
                             req_to_install.source_dir = location
+                            req_to_install.run_egg_info()
                             if url and url.scheme in vcs.all_schemes:
-                                req_to_install.run_egg_info()
                                 req_to_install.archive(self.download_dir)
                         else:
                             req_to_install.source_dir = location
@@ -984,20 +1053,22 @@ class RequirementSet(object):
                         # repeat check_if_exists to uninstall-on-upgrade (#14)
                         req_to_install.check_if_exists()
                         if req_to_install.satisfied_by:
-                            if self.upgrade:
+                            if self.upgrade or self.ignore_installed:
                                 req_to_install.conflicts_with = req_to_install.satisfied_by
                                 req_to_install.satisfied_by = None
                             else:
                                 install = False
-                if not is_bundle and not self.is_download:
+                if not is_bundle:
                     ## FIXME: shouldn't be globally added:
                     finder.add_dependency_links(req_to_install.dependency_links)
-                    ## FIXME: add extras in here:
+                    if (req_to_install.extras):
+                        logger.notify("Installing extra requirements: %r" % ','.join(req_to_install.extras))
                     if not self.ignore_dependencies:
-                        for req in req_to_install.requirements():
+                        for req in req_to_install.requirements(req_to_install.extras):
                             try:
                                 name = pkg_resources.Requirement.parse(req).project_name
-                            except ValueError, e:
+                            except ValueError:
+                                e = sys.exc_info()[1]
                                 ## FIXME: proper warning
                                 logger.error('Invalid requirement: %r (%s) in requirement %s' % (req, e, req_to_install))
                                 continue
@@ -1009,8 +1080,11 @@ class RequirementSet(object):
                             self.add_requirement(subreq)
                     if req_to_install.name not in self.requirements:
                         self.requirements[req_to_install.name] = req_to_install
+                    if self.is_download or req_to_install._temp_build_dir is not None:
+                        self.reqs_to_cleanup.append(req_to_install)
                 else:
                     self.reqs_to_cleanup.append(req_to_install)
+
                 if install:
                     self.successfully_downloaded.append(req_to_install)
                     if bundle and (req_to_install.url and req_to_install.url.startswith('file:///')):
@@ -1030,6 +1104,7 @@ class RequirementSet(object):
             remove_dir.append(self.build_dir)
 
         # The source dir of a bundle can always be removed.
+        # FIXME: not if it pre-existed the bundle!
         if bundle:
             remove_dir.append(self.src_dir)
 
@@ -1046,30 +1121,36 @@ class RequirementSet(object):
 
     def copy_to_build_dir(self, req_to_install):
         target_dir = req_to_install.editable and self.src_dir or self.build_dir
-        logger.info("Copying %s to %s" %(req_to_install.name, target_dir))
+        logger.info("Copying %s to %s" % (req_to_install.name, target_dir))
         dest = os.path.join(target_dir, req_to_install.name)
-        copytree(req_to_install.source_dir, dest)
-        call_subprocess(["python", "%s/setup.py"%dest, "clean"])
+        shutil.copytree(req_to_install.source_dir, dest)
+        call_subprocess(["python", "%s/setup.py" % dest, "clean"], cwd=dest,
+                        command_desc='python setup.py clean')
 
     def unpack_url(self, link, location, only_download=False):
         if only_download:
-            location = self.download_dir
+            loc = self.download_dir
+        else:
+            loc = location
         if is_vcs_url(link):
-            return unpack_vcs_link(link, location, only_download)
+            return unpack_vcs_link(link, loc, only_download)
         elif is_file_url(link):
-            return unpack_file_url(link, location)
+            return unpack_file_url(link, loc)
         else:
             if self.download_cache:
                 self.download_cache = os.path.expanduser(self.download_cache)
-            return unpack_http_url(link, location, self.download_cache, only_download)
+            retval = unpack_http_url(link, location, self.download_cache, self.download_dir)
+            if only_download:
+                _write_delete_marker_message(os.path.join(location, PIP_DELETE_MARKER_FILENAME))
+            return retval
 
     def install(self, install_options, global_options=()):
         """Install everything in this set (after having downloaded and unpacked the packages)"""
         to_install = [r for r in self.requirements.values()
-                      if self.upgrade or not r.satisfied_by]
+                      if not r.satisfied_by]
 
         if to_install:
-            logger.notify('Installing collected packages: %s' % (', '.join([req.name for req in to_install])))
+            logger.notify('Installing collected packages: %s' % ', '.join([req.name for req in to_install]))
         logger.indent += 2
         try:
             for requirement in to_install:
@@ -1189,7 +1270,7 @@ _scheme_re = re.compile(r'^(http|https|file):', re.I)
 
 def parse_requirements(filename, finder=None, comes_from=None, options=None):
     skip_match = None
-    skip_regex = options.skip_requirements_regex
+    skip_regex = options.skip_requirements_regex if options else None
     if skip_regex:
         skip_match = re.compile(skip_regex)
     filename, content = get_file_content(filename, comes_from=comes_from)
@@ -1207,7 +1288,7 @@ def parse_requirements(filename, finder=None, comes_from=None, options=None):
                 req_url = line[len('--requirement'):].strip().strip('=')
             if _scheme_re.search(filename):
                 # Relative to a URL
-                req_url = urlparse.urljoin(req_url, filename)
+                req_url = urlparse.urljoin(filename, req_url)
             elif not _scheme_re.search(req_url):
                 req_url = os.path.join(os.path.dirname(filename), req_url)
             for item in parse_requirements(req_url, finder, comes_from=filename, options=options):
@@ -1242,7 +1323,7 @@ def parse_requirements(filename, finder=None, comes_from=None, options=None):
                 if line.startswith('-e'):
                     line = line[2:].strip()
                 else:
-                    line = line[len('--editable'):].strip()
+                    line = line[len('--editable'):].strip().lstrip('=')
                 req = InstallRequirement.from_editable(
                     line, comes_from=comes_from, default_vcs=options.default_vcs)
             else:
@@ -1253,12 +1334,30 @@ def parse_requirements(filename, finder=None, comes_from=None, options=None):
 def parse_editable(editable_req, default_vcs=None):
     """Parses svn+http://blahblah@rev#egg=Foobar into a requirement
     (Foobar) and a URL"""
+
     url = editable_req
-    if os.path.isdir(url) and os.path.exists(os.path.join(url, 'setup.py')):
+    extras = None
+
+    # If a file path is specified with extras, strip off the extras.
+    m = re.match(r'^(.+)(\[[^\]]+\])$', url)
+    if m:
+        url_no_extras = m.group(1)
+        extras = m.group(2)
+    else:
+        url_no_extras = url
+
+    if os.path.isdir(url_no_extras):
+        if not os.path.exists(os.path.join(url_no_extras, 'setup.py')):
+            raise InstallationError("Directory %r is not installable. File 'setup.py' not found.", url_no_extras)
         # Treating it as code that has already been checked out
-        url = path_to_url(url)
-    if url.lower().startswith('file:'):
-        return None, url
+        url_no_extras = path_to_url(url_no_extras)
+
+    if url_no_extras.lower().startswith('file:'):
+        if extras:
+            return None, url_no_extras, pkg_resources.Requirement.parse('__placeholder__' + extras).extras
+        else:
+            return None, url_no_extras, None
+
     for version_control in vcs:
         if url.lower().startswith('%s:' % version_control):
             url = '%s+%s' % (version_control, url)
@@ -1270,8 +1369,10 @@ def parse_editable(editable_req, default_vcs=None):
                 '--editable=%s should be formatted with svn+URL, git+URL, hg+URL or bzr+URL' % editable_req)
     vc_type = url.split('+', 1)[0].lower()
     if not vcs.get_backend(vc_type):
-        raise InstallationError(
-            'For --editable=%s only svn (svn+URL), Git (git+URL), Mercurial (hg+URL) and Bazaar (bzr+URL) is currently supported' % editable_req)
+        error_message = 'For --editable=%s only ' % editable_req + \
+            ', '.join([backend.name + '+URL' for backend in vcs.backends]) + \
+            ' is currently supported'
+        raise InstallationError(error_message)
     match = re.search(r'(?:#|#.*?&)egg=([^&]*)', editable_req)
     if (not match or not match.group(1)) and vcs.get_backend(vc_type):
         parts = [p for p in editable_req.split('#', 1)[0].split('/') if p]
@@ -1290,7 +1391,7 @@ def parse_editable(editable_req, default_vcs=None):
     if match:
         # Strip off -dev, -0.2, etc.
         req = match.group(1)
-    return req, url
+    return req, url, None
 
 
 class UninstallPathSet(object):
@@ -1328,6 +1429,11 @@ class UninstallPathSet(object):
         else:
             self._refuse.add(path)
 
+        # __pycache__ files can show up after 'installed-files.txt' is created, due to imports
+        if os.path.splitext(path)[1] == '.py' and uses_pycache:
+            self.add(imp.cache_from_source(path))
+
+
     def add_pth(self, pth_file, entry):
         pth_file = normalize_path(pth_file)
         if self._permitted(pth_file):
@@ -1357,6 +1463,8 @@ class UninstallPathSet(object):
     def remove(self, auto_confirm=False):
         """Remove paths in ``self.paths`` with confirmation (unless
         ``auto_confirm`` is True)."""
+        if not self.paths:
+            raise InstallationError("Can't uninstall '%s'. No files were found to uninstall." % self.dist.project_name)
         if not self._can_uninstall():
             return
         logger.notify('Uninstalling %s:' % self.dist.project_name)
@@ -1429,19 +1537,21 @@ class UninstallPthEntries(object):
 
     def remove(self):
         logger.info('Removing pth entries from %s:' % self.file)
-        fh = open(self.file, 'r')
+        fh = open(self.file, 'rb')
+        # windows uses '\r\n' with py3k, but uses '\n' with py2.x
         lines = fh.readlines()
         self._saved_lines = lines
         fh.close()
-        try:
-            for entry in self.entries:
-                logger.info('Removing entry: %s' % entry)
+        if any(b('\r\n') in line for line in lines):
+            endline = '\r\n'
+        else:
+            endline = '\n'
+        for entry in self.entries:
             try:
-                lines.remove(entry + '\n')
+                logger.info('Removing entry: %s' % entry)
+                lines.remove(b(entry + endline))
             except ValueError:
                 pass
-        finally:
-            pass
         fh = open(self.file, 'wb')
         fh.writelines(lines)
         fh.close()
@@ -1465,6 +1575,12 @@ class FakeFile(object):
 
     def readline(self):
         try:
-            return self._gen.next()
+            try:
+                return next(self._gen)
+            except NameError:
+                return self._gen.next()
         except StopIteration:
             return ''
+
+    def __iter__(self):
+        return self._gen
